﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;

using NetworkCommsDotNet;
using NetworkCommsDotNet.Connections;
using NetworkCommsDotNet.DPSBase;
using NetworkCommsDotNet.Tools;

namespace TestProject
{
    /// <summary>
    /// 채팅 서버
    /// </summary>
    public class ChatServer : ChatServerBase
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 채팅 히스토리 텍스트 블럭 - ChatHistoryTextBlock

        /// <summary>
        /// 채팅 히스토리 텍스트 블럭
        /// </summary>
        public TextBlock ChatHistoryTextBlock { get; private set; }

        #endregion
        #region 채팅 히스토리 스크롤러 뷰어 - ChatHistoryScrollerViewer

        /// <summary>
        /// 채팅 히스토리 스크롤러 뷰어
        /// </summary>
        public ScrollViewer ChatHistoryScrollerViewer { get; private set; }

        #endregion
        #region 메시지 발송자 텍스트 박스 - MessagesSenderTextBox

        /// <summary>
        /// 메시지 발송자 텍스트 박스
        /// </summary>
        public TextBox MessagesSenderTextBox { get; private set; }

        #endregion
        #region 메시지 텍스트 박스 - MessagesTextBox

        /// <summary>
        /// 메시지 텍스트 박스
        /// </summary>
        public TextBox MessagesTextBox { get; private set; }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ChatServer(chatHistoryTextBlock, chatHistoryScrollViewer, messageSenderTextBox, messageTextBox)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="chatHistoryTextBlock">채팅 히스토리 텍스트 박스</param>
        /// <param name="chatHistoryScrollViewer">채팅 히스토리 스크롤 뷰어</param>
        /// <param name="messageSenderTextBox">메시지 발송자 텍스트 박스</param>
        /// <param name="messageTextBox">메시지 텍스트 박스</param>
        public ChatServer
        (
            TextBlock    chatHistoryTextBlock,
            ScrollViewer chatHistoryScrollViewer,
            TextBox      messageSenderTextBox,
            TextBox messageTextBox
        ) : base (HostInfo.HostName, ConnectionType.TCP)
        {
            this.ChatHistoryTextBlock      = chatHistoryTextBlock;
            this.ChatHistoryScrollerViewer = chatHistoryScrollViewer;
            this.MessagesSenderTextBox     = messageSenderTextBox;
            this.MessagesTextBox           = messageTextBox;
            this.DataSerializer            = DPSManager.GetDataSerializer<ProtobufSerializer>();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 채팅 히스토리 텍스트 블럭에 라인 추가하기 - AppendLineToChatHistoryTextBlock(fontStyle, text, addNewLine)

        /// <summary>
        /// 채팅 히스토리 텍스트 블럭에 라인 추가하기
        /// </summary>
        /// <param name="fontStyle">폰트 스타일</param>
        /// <param name="text">텍스트</param>
        /// <param name="addNewLine">신규 라인 추가 여부</param>
        public void AppendLineToChatHistoryTextBlock(System.Drawing.FontStyle fontStyle, string text, bool addNewLine)
        {
            ChatHistoryTextBlock.Dispatcher.BeginInvoke
            (
                new Action(() =>
                {
                    if(fontStyle == System.Drawing.FontStyle.Regular)
                    {
                        ChatHistoryTextBlock.Inlines.Add(new Run(text));
                    }
                    else if(fontStyle == System.Drawing.FontStyle.Bold)
                    {
                        ChatHistoryTextBlock.Inlines.Add(new Bold(new Run(text)));
                    }
                    else if(fontStyle == System.Drawing.FontStyle.Italic)
                    {
                        ChatHistoryTextBlock.Inlines.Add(new Italic(new Run(text)));
                    }
                    else
                    {
                        ChatHistoryTextBlock.Inlines.Add
                        (
                            new Run
                            (
                                "에러 : 알 수 없는 글꼴 스타일을 가진 알 수 없는 텍스트를 추가하려고 했습니다."
                            )
                        );
                    }

                    if(addNewLine)
                    {
                        ChatHistoryTextBlock.Inlines.Add("\n");
                    }

                    ChatHistoryScrollerViewer.ScrollToBottom();
                })
            );
        }

        #endregion
        #region 메시지 발송자 텍스트 박스 갱신하기 - UpdateMessageSenderTextBox()

        /// <summary>
        /// 메시지 발송자 텍스트 박스 갱신하기
        /// </summary>
        public void UpdateMessageSenderTextBox()
        {
            lock(this.chatMessageDictionary)
            {
                string[] sourceNameArray =
                (
                    from    current in chatMessageDictionary.Values
                    orderby current.SourceName
                    select  current.SourceName
                ).ToArray();

                MessagesSenderTextBox.Dispatcher.BeginInvoke
                (
                    new Action<string[]>(userNameArray =>
                    {
                        MessagesSenderTextBox.Text = string.Empty;

                        foreach(string userName in userNameArray)
                        {
                            MessagesSenderTextBox.AppendText(userName + "\n");
                        }
                    }),
                    new object[] { sourceNameArray }
                );
            }
        }

        #endregion

        #region 채팅 히스토리에 라인 추가하기 - AppendLineToChatHistory(message)

        /// <summary>
        /// 채팅 히스토리에 라인 추가하기
        /// </summary>
        /// <param name="message">메시지</param>
        public override void AppendLineToChatHistory(string message)
        {
            AppendLineToChatHistoryTextBlock(System.Drawing.FontStyle.Regular, message, true);
        }

        #endregion
        #region 채팅 히스토리 지우기 - ClearChatHistory()

        /// <summary>
        /// 채팅 히스토리 지우기
        /// </summary>
        public override void ClearChatHistory()
        {
            ChatHistoryTextBlock.Dispatcher.BeginInvoke
            (
                new Action(() =>
                {
                    ChatHistoryTextBlock.Inlines.Clear();

                    ChatHistoryScrollerViewer.ScrollToBottom();
                })
            );
        }

        #endregion
        #region 입력 라인 지우기 - ClearInputLine()

        /// <summary>
        /// 입력 라인 지우기
        /// </summary>
        public override void ClearInputLine()
        {
            MessagesTextBox.Dispatcher.BeginInvoke
            (
                new Action(() =>
                {
                    MessagesTextBox.Text = string.Empty;
                })
            );
        }

        #endregion
        #region 메시지 보여주기 - ShowMessage(message)

        /// <summary>
        /// 메시지 보여주기
        /// </summary>
        /// <param name="message">메시지</param>
        public override void ShowMessage(string message)
        {
            MessageBox.Show(message);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 채팅 메시지 수신시 처리하기 - ProcessChatMessageReceived(packetHeader, connection, chatMessage)

        /// <summary>
        /// 채팅 메시지 수신시 처리하기
        /// </summary>
        /// <param name="packetHeader">패킷 헤더</param>
        /// <param name="connection">연결</param>
        /// <param name="chatMessage">채팅 메시지</param>
        protected override void ProcessChatMessageReceived
        (
            PacketHeader packetHeader,
            Connection   connection,
            ChatMessage  chatMessage
        )
        {
            base.ProcessChatMessageReceived(packetHeader, connection, chatMessage);

            UpdateMessageSenderTextBox();
        }

        #endregion
    }
}