﻿using System.ComponentModel;
using System.Windows;
using System.Windows.Input;

using NetworkCommsDotNet;
using NetworkCommsDotNet.Connections;
using NetworkCommsDotNet.DPSBase;
using NetworkCommsDotNet.Tools;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 채팅 서버
        /// </summary>
        private ChatServer charServer;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            this.localNameTextBox.Text = HostInfo.HostName;

            this.charServer = new ChatServer
            (
                this.chatMessageTextBlock,
                this.chatMessageScrollViewer,
                this.messageSenderTextBox,
                this.messageTextBox
            );

            this.charServer.PrintUsageInstructions();

            this.charServer.RefreshConfiguration();

            Closing                                  += Window_Closing;
            this.serverIPTextBox.LostFocus           += serverIPTextBox_LostFocus;
            this.serverPortTextBox.LostFocus         += serverPortTextBox_LostFocus;
            this.localNameTextBox.LostFocus          += localNameTextBox_LostFocus;
            this.enableLocalServerCheckBox.Checked   += enableLocalServerCheckBox_Checked;
            this.enableLocalServerCheckBox.Unchecked += enableLocalServerCheckBox_Checked;
            this.tcpRadioButton.Checked              += tcpRadioButton_Checked;
            this.udpRadioButton.Checked              += udpRadioButton_Checked;
            this.protobufRadiobButton.Checked        += protobufRadiobButton_Checked;
            this.jsonRadioButton.Checked             += jsonRadioButton_Checked;
            this.useEncryptionCheckBox.Checked       += useEncryptionCheckBox_Checked;
            this.useEncryptionCheckBox.Unchecked     += useEncryptionCheckBox_Checked;
            this.messageTextBox.KeyUp                += messageTextBox_KeyUp;
            this.sendMessageButton.Click             += sendMessageButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 윈도우 닫을 경우 처리하기 - Window_Closing(sender, e)

        /// <summary>
        /// 윈도우 닫을 경우 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Window_Closing(object sender, CancelEventArgs e)
        {
            NetworkComms.Shutdown();
        }

        #endregion
        #region 서버 IP 텍스트 박스 포커스 상실시 처리하기 - serverIPTextBox_LostFocus(sender, e)

        /// <summary>
        /// 서버 IP 텍스트 박스 포커스 상실시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void serverIPTextBox_LostFocus(object sender, RoutedEventArgs e)
        {
            this.charServer.ServerIPAddress = this.serverIPTextBox.Text.Trim();
        }

        #endregion
        #region 서버 포트 텍스트 박스 포커스 상실시 처리하기 - serverPortTextBox_LostFocus(sender, e)

        /// <summary>
        /// 서버 포트 텍스트 박스 포커스 상실시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void serverPortTextBox_LostFocus(object sender, RoutedEventArgs e)
        {
            int portNumber;

            if(int.TryParse(this.serverPortTextBox.Text, out portNumber))
            {
                this.charServer.ServerPort = portNumber;
            }
            else
            {
                this.serverPortTextBox.Text = string.Empty;
            }
        }

        #endregion
        #region 로컬명 텍스트 박스 포커스 상실시 처리하기 - localNameTextBox_LostFocus(sender, e)

        /// <summary>
        /// 로컬명 텍스트 박스 포커스 상실시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void localNameTextBox_LostFocus(object sender, RoutedEventArgs e)
        {
            this.charServer.LocalName = this.localNameTextBox.Text.Trim();
        }

        #endregion
        #region 로컬 서버 활성화 체크 박스 체크시 처리하기 - enableLocalServerCheckBox_Checked(sender, e)

        /// <summary>
        /// 로컬 서버 활성화 체크 박스 체크시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void enableLocalServerCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            this.charServer.LocalServerEnabled = (bool)enableLocalServerCheckBox.IsChecked;

            this.charServer.RefreshConfiguration();

            this.protobufRadiobButton.IsEnabled = !this.charServer.LocalServerEnabled;
            this.jsonRadioButton.IsEnabled      = !this.charServer.LocalServerEnabled;
        }

        #endregion
        #region TCP 라디오 버튼 체크시 처리하기 - tcpRadioButton_Checked(sender, e)

        /// <summary>
        /// TCP 라디오 버튼 체크시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void tcpRadioButton_Checked(object sender, RoutedEventArgs e)
        {
            if(this.udpRadioButton != null && this.udpRadioButton.IsChecked != null && !(bool)this.udpRadioButton.IsChecked)
            {
                this.tcpRadioButton.IsChecked = true;
                this.udpRadioButton.IsChecked = false;

                this.charServer.ConnectionType = ConnectionType.TCP;

                this.charServer.RefreshConfiguration();
            }
        }

        #endregion
        #region UDP 라디오 버튼 체크시 처리하기 - udpRadioButton_Checked(sender, e)

        /// <summary>
        /// UDP 라디오 버튼 체크시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void udpRadioButton_Checked(object sender, RoutedEventArgs e)
        {
            if(this.tcpRadioButton != null && this.tcpRadioButton.IsChecked != null && !(bool)this.tcpRadioButton.IsChecked)
            {
                this.tcpRadioButton.IsChecked = false;
                this.udpRadioButton.IsChecked = true;

                this.charServer.ConnectionType = ConnectionType.UDP;

                this.charServer.RefreshConfiguration();
            }
        }

        #endregion
        #region Protobuf 라디오 버튼 체크시 처리하기 - protobufRadiobButton_Checked(sender, e)

        /// <summary>
        /// Protobuf 라디오 버튼 체크시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void protobufRadiobButton_Checked(object sender, RoutedEventArgs e)
        {
            if(this.jsonRadioButton != null && this.jsonRadioButton.IsChecked != null && !(bool)this.jsonRadioButton.IsChecked)
            {
                this.protobufRadiobButton.IsChecked = true;
                this.jsonRadioButton.IsChecked      = false;

                this.charServer.DataSerializer = DPSManager.GetDataSerializer<ProtobufSerializer>();

                this.charServer.RefreshConfiguration();

                this.charServer.AppendLineToChatHistory("데이터 직렬화기가 Protobuf 직렬화기로 변경되었습니다.");
            }
        }

        #endregion
        #region JSON 라디오 버튼 체크시 처리하기 - jsonRadioButton_Checked(sender, e)

        /// <summary>
        /// JSON 라디오 버튼 체크시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void jsonRadioButton_Checked(object sender, RoutedEventArgs e)
        {
            if
            (
                this.protobufRadiobButton           != null &&
                this.protobufRadiobButton.IsChecked != null &&
                !(bool)this.protobufRadiobButton.IsChecked
            )
            {
                this.protobufRadiobButton.IsChecked = false;
                this.jsonRadioButton.IsChecked      = true;

                this.charServer.DataSerializer = DPSManager.GetDataSerializer<JSONSerializer>();

                this.charServer.RefreshConfiguration();

                this.charServer.AppendLineToChatHistory("데이터 직렬화기가 JSON 직렬화기로 변경되었습니다.");
            }
        }

        #endregion
        #region 암호화 사용 체크 박스 체크시 처리하기 - useEncryptionCheckBox_Checked(sender, e)

        /// <summary>
        /// 암호화 사용 체크 박스 체크시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void useEncryptionCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            this.charServer.EncryptionEnabled = (bool)useEncryptionCheckBox.IsChecked;

            this.charServer.RefreshConfiguration();
        }

        #endregion
        #region 메시지 텍스트 박스 키 UP시 처리하기 - messageTextBox_KeyUp(sender, e)

        /// <summary>
        /// 메시지 텍스트 박스 키 UP시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void messageTextBox_KeyUp(object sender, KeyEventArgs e)
        {
            if(e.Key == Key.Enter || e.Key == Key.Return)
            {
                this.charServer.SendMessage(this.messageTextBox.Text);
            }
        }

        #endregion
        #region 메시지 발송 버튼 클릭시 처리하기 - sendMessageButton_Click(sender, e)

        /// <summary>
        /// 메시지 발송 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void sendMessageButton_Click(object sender, RoutedEventArgs e)
        {
            this.charServer.SendMessage(messageTextBox.Text);
        }

        #endregion
    }
}